## Programm um DMM ME-42 von Metex (Voltcraft) mit GUI auszulesen und Messreihe grafisch darzustellen.
## Beliebiges Byte senden: DMM gibt aktuellen Messwert mit Einheit und Messgroeße zurueck.
## Bibliothek statistics, tkinter und matplotlib noetig.
## Baudrate maximal 600, 7 Bit Daten, keine Paritaet, 2 Stoppbits, DTR eingeschaltet und RTS ausgeschaltet.
## Rueckgabe von 14 Byte Messdaten. Farben siehe https://wiki.tcl.tk/37701
## To Dos: Abspeichern Messreihe in Datei, Anzeigen Messmodus AC bzw. DC, Abfangen Fehler
## bei Overflow Rueckgabewert des DMM.
## Serial siehe: pythonhosted.org/pyserial/shortintro.html#opening-serial-ports
## 12.3.18 S. Mack

#!/usr/bin/python3

import matplotlib.pyplot as plt
import tkinter as TK
import serial
from time import sleep
from statistics import mean
from statistics import stdev

# Objekt fuer serielle Kommunikation
portDmm = serial.Serial()

# Tkinter Hauptfenster
root=TK.Tk()
root.title("Multimeter ME-42")

# Eingabewerte durch Benutzer
comPortNo = TK.StringVar() # StringVar()-Objekt wird erzeugt und der Variablen comPortNo zugewiesen
comPortNo.set("COM42") # Initalwerte werden dem Objekt gegeben
numMeas = TK.IntVar() # IntVar()-Objekt wird erzeugt und der Variablen numMeas zugewiesen
numMeas.set(20) # Initalwerte werden dem Objekt gegeben
sampTime = TK.IntVar()
sampTime.set(1)

# GUI Texte (macht hier Sinn, wenn andere Sprache noetig
comPortTxt='COM-Port:'
numMeasPerTxt='# Messungen / Periode (s):'
valTxt='Aktueller Messwert:'
unitTxt='Einheit'
valMeanTxt='Mittelwert:'
valStdDevTxt='Standardabweichung:'

# COM-Verbindung aufbauen
def connComPort(port):
    print('connComPort()')
    # Serielle Schnittstelle oeffnen. Muss vor GUI geschehen, damit dort Info ueber COM-Verbindungsstatus
    try:
        port.baudrate = 600
        port.port = comPortNo.get()
        port.parity = serial.PARITY_NONE
        port.stopbit = serial.STOPBITS_TWO
        port.bytesize = serial.SEVENBITS
        port.timeout = 2        
        port.setDTR(True) # fuer Stromversorgung RS-232 Interface.
        port.setRTS(False)
        port.open()
        print('COM-Verbindung OK')
        comStatusLabel.configure(text='COM-Verbindung OK')
        port.flushInput() # Seriellen Puffer leeren
    except:
        print('Keine COM-Verbindung')
        comStatusLabel.configure(text='Keine COM-Verbindung')

# Call Back Funktion fuer Klicken des Buttons (in tkinter kann leider kein Parameter uebergeben werden)
def comConnButton():
    print('comConnButton()')
    connComPort(portDmm)
    
# Call Back Funktion fuer Klicken des Buttons (in tkinter kann leider kein Parameter uebergeben werden)
def singleMeasButton():
    print('singleMeasButton()')
    singleMeas(portDmm)

# Call Back Funktion fuer Klicken des Buttons (in tkinter kann leider kein Parameter uebergeben werden)
def multiMeasPlotButton():
	multiMeasPlot(portDmm)

# Einzelmessung
def singleMeas(port):
    print('Einzelmessung')
    try:
        port.write(b"D") # port.write akzeptiert nur Byte-Werte, DMM misst egal welches Zeichen
        result = port.read(14) # Daten aus dem DMM auslesen
        if len(result) == 14: # Umwandlung in String und Formatierungszeichen entfernen, falls Datensatz konsistent
            print(str(result)[13:15])
            unit = str(result)[13:15]
            mode = str(result)[2:4]
            val = str(result)[5:11]
            comStatusLabel.configure(text='Multimeter OK') # Labeltext aktualisieren
        else:
            print('Kein passender Antwortstring vom Multimeter (Multimeter antwortet nicht).')
            val = '00000' # Muss Zahl sein, falls Messreihe (wegen Berechnung Statistikparameter)
            comStatusLabel.configure(text='Daten DMM inkonsistent')
    except:
        print('COM-Verbindung abgebrochen')
        comStatusLabel.configure(text='COM-Verbindung abgebrochen')
        val = '00000'
    valLabel.configure(text=val)
    unitLabel.configure(text=unit+mode)
    return val
    
# Messreihe    
def multiMeasPlot(port):
    print('Messreihe')
    results = list(range(0, numMeas.get())) # Liste fuer Messwerte, muesste eigentlich nicht initialisiert werden
    times = [j * sampTime.get() for j in list(range(0, numMeas.get()))] # Messzeiten als "List Comprehension"
    for i in range(0,numMeas.get(),1):
        results[i] = float(singleMeas(port))
        print('Messreihe Messung ', i)
        sleep(sampTime.get()) # Samplingperiode lang warten
    print('Mittelwert: ', mean(results), 'Standardabweichung: ', stdev(results))
    meanLabel.configure(text=mean(results))
    stdevLabel.configure(text=stdev(results))
    xwerte = [j * sampTime.get() for j in list(range(0, numMeas.get()))] 
    plt.plot(times, results)
    plt.xlabel('Zeit in Sekunden')
    plt.ylabel(unitLabel.cget('text')) # Umweg ueber Label, da keine globalen Variablen
    plt.title('DMM ME-42: Ergebnis '+str(numMeas.get()) + ' Messungen')
    plt.xticks(xwerte) # x-Achsenbeschriftung fuer jeden Messwert
    plt.grid(True)
    plt.show()
       

# Einzelne widgets: Hier werden die Buttons, Labels und Eingabefelder erzeugt und deren Layout
# festgelegt. Bei den Eingabefeldern (TK.Entry) werden die Variablen definiert.
comConnButton=TK.Button(root,text="COM-Schnittstelle verbinden",command=comConnButton,bg='light blue') # Button COM Verbinden
comStatusLabel=TK.Label(root,text='Keine COM-Verbindung',width=20,bg='light blue') # Status COM Verbindung

comInputLabel=TK.Label(root,text=comPortTxt,width=len(comPortTxt)) # Anzahl Messungen / Abtastperiode
comPortEnter=TK.Entry(root,textvariable=comPortNo,width=len(comPortTxt)) # Eingabemaske fuer COM-Port Nummer

numSampInputLabel=TK.Label(root,text=numMeasPerTxt,width=len(numMeasPerTxt))
numMeasEnter=TK.Entry(root,textvariable=numMeas,width=8) # Eingabemaske fuer Anzahl Messungen
sampTimeEnter=TK.Entry(root,textvariable=sampTime,width=8) # Eingabemaske fuer Abtastperiode

singleMeasButton=TK.Button(root,text="Einzelmessung",command=singleMeasButton) # Button Einzelmessung
multiMeasButton=TK.Button(root,text="Messreihe+Plot",command=multiMeasPlotButton) # Button Messreihe

valTextLabel=TK.Label(root,text=valTxt,width=len(valTxt)) # Aktueller Messwert
valLabel=TK.Label(root,text='-----',width=20) # Ausgabe aktueller Messwert

unitTextLabel=TK.Label(root,text=unitTxt,width=len(unitTxt)) # Akutelle Einheit
unitLabel=TK.Label(root,text='-----',width=20) # Ausgabe Einheit

meanTextLabel=TK.Label(root,text=valMeanTxt,width=len(valMeanTxt)) # Mittelwert Messreihe
meanLabel=TK.Label(root,text='-----',width=20) # Ausgabe Mittelwert

stdevTextLabel=TK.Label(root,text=valStdDevTxt,width=len(valStdDevTxt)) # Standardabweichung Messreihe
stdevLabel=TK.Label(root,text='-----',width=20) # Ausgabe Standardabweichung

#Festlegung Layout der gesamten GUI: Tabellenanordnung der Widget TK.E = Textausrichtung East
comConnButton.grid(row=0,columnspan=3,sticky=TK.E+TK.W) 
comStatusLabel.grid(row=1,columnspan=3,sticky=TK.E+TK.W)

comInputLabel.grid(row=2,column=0,sticky=TK.E)
comPortEnter.grid(row=2,column=1,sticky=TK.W)

numSampInputLabel.grid(row=3,column=0,sticky=TK.E) 
numMeasEnter.grid(row=3,column=1,sticky=TK.W)
sampTimeEnter.grid(row=3,column=2,sticky=TK.W)

singleMeasButton.grid(row=4,column=0,sticky=TK.E)
multiMeasButton.grid(row=4,column=1,sticky=TK.W)

valTextLabel.grid(row=5,column=0,sticky=TK.E)
valLabel.grid(row=5,column=1,sticky=TK.W)

unitTextLabel.grid(row=6,column=0,sticky=TK.E)
unitLabel.grid(row=6,column=1,sticky=TK.W)

meanTextLabel.grid(row=7,column=0,sticky=TK.E)
meanLabel.grid(row=7,column=1,sticky=TK.W)

stdevTextLabel.grid(row=8,column=0,sticky=TK.E)
stdevLabel.grid(row=8,column=1,sticky=TK.W)

# GUI starten
TK.mainloop()


